@@ -1,6 +1,5 @@
-/* -*- mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/*
- * gcal-ediat-dialog.c
+/* gcal-ediat-dialog.c
+ *
  * Copyright (C) 2015 Erick Pérez Castellanos <erickpc@gnome.org>
  *
  * gnome-calendar is free software: you can redistribute it and/or modify it
@@ -108,7 +107,7 @@ struct _GcalEditDialog
   ESource          *selected_source;
 
   /* flags */
-  gboolean          format_24h;
+  GcalTimeFormat    time_format;
   gboolean          event_is_new;
   gboolean          recurrence_changed;
   gboolean          setting_event;
@@ -128,11 +127,14 @@ enum
   PROP_0,
   PROP_EVENT,
   PROP_MANAGER,
+  PROP_TIME_FORMAT,
   PROP_WRITABLE,
-  LAST_PROP
+  N_PROPS
 };
 
-const GActionEntry action_entries[] =
+static GParamSpec* properties[N_PROPS] = { NULL, };
+
+static const GActionEntry action_entries[] =
 {
   { "select-calendar", on_calendar_selected, "s" },
 };
@@ -276,7 +278,7 @@ gcal_edit_dialog_set_writable (GcalEditDialog *dialog,
 
   dialog->writable = writable;
 
-  g_object_notify (G_OBJECT (dialog), "writable");
+  g_object_notify_by_pspec (G_OBJECT (dialog), properties[PROP_WRITABLE]);
 }
 
 static void
@@ -452,6 +454,7 @@ action_button_clicked (GtkWidget *widget,
       GcalRecurrenceFrequency freq;
       GcalRecurrence *old_recur;
       GDateTime *start_date, *end_date;
+      gboolean was_all_day;
       gboolean all_day;
       gchar *note_text;
 
@@ -468,6 +471,7 @@ action_button_clicked (GtkWidget *widget,
 
       /* Update all day */
       all_day = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->all_day_check));
+      was_all_day = gcal_event_get_all_day (dialog->event);
 
       gcal_event_set_all_day (dialog->event, all_day);
 
@@ -483,7 +487,7 @@ action_button_clicked (GtkWidget *widget,
 
       /*
        * Update start & end dates. The dates are already translated to the current
-       * timezone.
+       * timezone (unless the event used to be all day, but no longer is).
        */
       start_date = gcal_edit_dialog_get_date_start (dialog);
       end_date = gcal_edit_dialog_get_date_end (dialog);
@@ -500,6 +504,23 @@ action_button_clicked (GtkWidget *widget,
           g_clear_pointer (&end_date, g_date_time_unref);
           end_date = fake_end_date;
         }
+      else if (!all_day && was_all_day)
+        {
+          /* When an all day event is changed to be not an all day event, we
+           * need to correct for the fact that the event's timezone was until
+           * now set to UTC. That means we need to change the timezone to
+           * localtime now, or else it will be saved incorrectly.
+           */
+          GDateTime *localtime_date;
+
+          localtime_date = g_date_time_to_local (start_date);
+          g_clear_pointer (&start_date, g_date_time_unref);
+          start_date = localtime_date;
+
+          localtime_date = g_date_time_to_local (end_date);
+          g_clear_pointer (&end_date, g_date_time_unref);
+          end_date = localtime_date;
+        }
 
       gcal_event_set_date_start (dialog->event, start_date);
       gcal_event_set_date_end (dialog->event, end_date);
@@ -700,6 +721,10 @@ gcal_edit_dialog_get_property (GObject    *object,
       g_value_set_object (value, self->manager);
       break;
 
+    case PROP_TIME_FORMAT:
+      g_value_set_enum (value, self->time_format);
+      break;
+
     case PROP_WRITABLE:
       g_value_set_boolean (value, self->writable);
       break;
@@ -727,6 +752,10 @@ gcal_edit_dialog_set_property (GObject      *object,
       gcal_edit_dialog_set_manager (self, g_value_get_object (value));
       break;
 
+    case PROP_TIME_FORMAT:
+      gcal_edit_dialog_set_time_format (self, g_value_get_enum (value));
+      break;
+
     case PROP_WRITABLE:
       gcal_edit_dialog_set_writable (self, g_value_get_boolean (value));
       break;
@@ -755,39 +784,47 @@ gcal_edit_dialog_class_init (GcalEditDialogClass *klass)
    *
    * The #GcalEvent being edited.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_EVENT,
-                                   g_param_spec_object ("event",
-                                                        "event of the dialog",
-                                                        "The event being edited",
-                                                        GCAL_TYPE_EVENT,
-                                                        G_PARAM_READWRITE));
+  properties[PROP_EVENT] = g_param_spec_object ("event",
+                                                "event of the dialog",
+                                                "The event being edited",
+                                                GCAL_TYPE_EVENT,
+                                                G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * GcalEditDialog::manager:
    *
    * The #GcalManager of the dialog.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_MANAGER,
-                                   g_param_spec_object ("manager",
-                                                        "Manager of the dialog",
-                                                        "The manager of the dialog",
-                                                        GCAL_TYPE_MANAGER,
-                                                        G_PARAM_READWRITE));
+  properties[PROP_MANAGER] = g_param_spec_object ("manager",
+                                                  "Manager of the dialog",
+                                                  "The manager of the dialog",
+                                                  GCAL_TYPE_MANAGER,
+                                                  G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * GcalEditDialog::time-format:
+   *
+   * The time format.
+   */
+  properties[PROP_TIME_FORMAT] = g_param_spec_enum ("time-format",
+                                                    "Manager of the dialog",
+                                                    "The manager of the dialog",
+                                                    GCAL_TYPE_TIME_FORMAT,
+                                                    GCAL_TIME_FORMAT_24H,
+                                                    G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * GcalEditDialog::writable:
    *
    * Whether the current event can be edited or not.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_WRITABLE,
-                                   g_param_spec_boolean ("writable",
-                                                         "Whether the current event can be edited",
-                                                         "Whether the current event can be edited or not",
-                                                         TRUE,
-                                                         G_PARAM_READWRITE));
+  properties[PROP_WRITABLE] = g_param_spec_boolean ("writable",
+                                                    "Whether the current event can be edited",
+                                                    "Whether the current event can be edited or not",
+                                                    TRUE,
+                                                    G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, N_PROPS, properties);
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/edit-dialog.ui");
 
@@ -1191,14 +1228,16 @@ gcal_edit_dialog_new (void)
  */
 void
 gcal_edit_dialog_set_time_format (GcalEditDialog *dialog,
-                                  gboolean        use_24h_format)
+                                  GcalTimeFormat  time_format)
 {
   g_return_if_fail (GCAL_IS_EDIT_DIALOG (dialog));
 
-  dialog->format_24h = use_24h_format;
+  dialog->time_format = time_format;
 
-  gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->start_time_selector), dialog->format_24h);
-  gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->end_time_selector), dialog->format_24h);
+  gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->start_time_selector), dialog->time_format);
+  gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (dialog->end_time_selector), dialog->time_format);
+
+  g_object_notify_by_pspec (G_OBJECT (dialog), properties[PROP_TIME_FORMAT]);
 }
 
 /**
@@ -1390,7 +1429,7 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
   setup_alarms (dialog);
 
 out:
-  g_object_notify (G_OBJECT (dialog), "event");
+  g_object_notify_by_pspec (G_OBJECT (dialog), properties[PROP_EVENT]);
 
   dialog->setting_event = FALSE;
 
@@ -1412,7 +1451,7 @@ gcal_edit_dialog_set_manager (GcalEditDialog *dialog,
   g_return_if_fail (GCAL_IS_MANAGER (manager));
 
   if (g_set_object (&dialog->manager, manager))
-    g_object_notify (G_OBJECT (dialog), "manager");
+    g_object_notify_by_pspec (G_OBJECT (dialog), properties[PROP_MANAGER]);
 }
 
 static GDateTime*
