@@ -1,6 +1,5 @@
-/* -*- mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/*
- * gcal-event-widget.c
+/* gcal-event-widget.c
+ *
  * Copyright (C) 2015 Erick Pérez Castellanos <erickpc@gnome.org>
  *
  * gnome-calendar is free software: you can redistribute it and/or modify it
@@ -19,8 +18,9 @@
 
 #define G_LOG_DOMAIN "GcalEventWidget"
 
-#include <string.h>
+#include <libecal/libecal.h>
 #include <glib/gi18n.h>
+#include <string.h>
 
 #include "gcal-event-widget.h"
 #include "gcal-utils.h"
@@ -38,10 +38,12 @@ struct _GcalEventWidget
   GDateTime          *dt_end;
 
   /* widgets */
-  GtkWidget          *ghost_grid;
+  GtkWidget          *color_box;
+  GtkWidget          *horizontal_grid;
   GtkWidget          *hour_label;
-  GtkWidget          *main_grid;
+  GtkWidget          *stack;
   GtkWidget          *summary_label;
+  GtkWidget          *vertical_grid;
 
   /* internal data */
   gboolean            clock_format_24h : 1;
@@ -54,7 +56,10 @@ struct _GcalEventWidget
 
   GtkOrientation      orientation;
 
-  guint               toggle_description_id;
+  guint               vertical_label_source_id;
+  gboolean            vertical_value_to_set;
+  gboolean            vertical_labels;
+
   gboolean            button_pressed;
 };
 
@@ -74,8 +79,6 @@ enum
   NUM_SIGNALS
 };
 
-
-
 typedef enum
 {
   CURSOR_NONE,
@@ -93,37 +96,53 @@ G_DEFINE_TYPE_WITH_CODE (GcalEventWidget, gcal_event_widget, GTK_TYPE_BIN,
  */
 
 static gboolean
-toggle_description_position_cb (GcalEventWidget *self)
+can_hold_vertical_labels_with_height (GcalEventWidget *self,
+                                      gint             height)
 {
-  gint current_top;
+  gint total_height;
 
-  gtk_container_child_get (GTK_CONTAINER (self->main_grid),
-                           self->summary_label,
-                           "top-attach", &current_top,
-                           NULL);
+  gtk_widget_get_preferred_height (self->vertical_grid, &total_height, NULL);
 
-  gtk_label_set_line_wrap (GTK_LABEL (self->summary_label),
-                           !gtk_label_get_line_wrap (GTK_LABEL (self->summary_label)));
+  return height >= total_height;
+}
 
-  gtk_container_child_set (GTK_CONTAINER (self->main_grid),
-                           self->summary_label,
-                           "left-attach", current_top == 1 ? 1 : 0,
-                           "top-attach", current_top == 1 ? 0 : 1,
-                           "width", current_top == 1 ? 1 : 2,
-                           NULL);
+static void
+set_vertical_labels (GcalEventWidget *self,
+                     gboolean         vertical)
+{
+  if (self->vertical_labels == vertical)
+    return;
 
-  self->toggle_description_id = 0;
+  gtk_stack_set_visible_child (GTK_STACK (self->stack), vertical ? self->vertical_grid : self->horizontal_grid);
+  self->vertical_labels = vertical;
+}
 
+static gboolean
+set_vertical_labels_in_idle_cb (gpointer data)
+{
+  GcalEventWidget *self = (GcalEventWidget*) data;
+
+  set_vertical_labels (self, self->vertical_value_to_set);
+
+  self->vertical_label_source_id = 0;
   return G_SOURCE_REMOVE;
 }
 
 static void
-queue_toggle_description_position (GcalEventWidget *self)
+queue_set_vertical_labels (GcalEventWidget *self,
+                           gboolean         vertical)
 {
-  if (self->toggle_description_id > 0)
-    g_source_remove (self->toggle_description_id);
+  if (self->vertical_label_source_id > 0)
+    {
+      g_source_remove (self->vertical_label_source_id);
+      self->vertical_label_source_id = 0;
+    }
 
-  self->toggle_description_id = g_idle_add ((GSourceFunc) toggle_description_position_cb, self);
+  if (self->vertical_labels == vertical)
+    return;
+
+  self->vertical_value_to_set = vertical;
+  self->vertical_label_source_id = g_idle_add (set_vertical_labels_in_idle_cb, self);
 }
 
 static void
@@ -153,7 +172,9 @@ static void
 gcal_event_widget_update_style (GcalEventWidget *self)
 {
   GtkStyleContext *context;
-  gboolean slanted_start, slanted_end;
+  gboolean slanted_start;
+  gboolean slanted_end;
+  gboolean timed;
 
   context = gtk_widget_get_style_context (GTK_WIDGET (self));
   slanted_start = FALSE;
@@ -183,8 +204,8 @@ gcal_event_widget_update_style (GcalEventWidget *self)
     gtk_style_context_add_class (context, "slanted-end");
 
   /* TODO: adjust margins based on the CSS gradients sizes, not hardcoded */
-  gtk_widget_set_margin_start (self->main_grid, slanted_start ? 20 : 4);
-  gtk_widget_set_margin_end (self->main_grid, slanted_end ? 20 : 4);
+  gtk_widget_set_margin_start (self->stack, slanted_start ? 20 : 4);
+  gtk_widget_set_margin_end (self->stack, slanted_end ? 20 : 4);
 
   /* Add style classes for orientation selectors */
   if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
@@ -202,17 +223,26 @@ gcal_event_widget_update_style (GcalEventWidget *self)
    * If the event is a timed, single-day event, draw it differently
    * from all-day or multi-day events.
    */
-  if (!gcal_event_get_all_day (self->event) && !gcal_event_is_multiday (self->event))
+  timed = !gcal_event_get_all_day (self->event) && !gcal_event_is_multiday (self->event);
+
+  gtk_widget_set_visible (self->color_box, timed);
+
+  if (timed)
     {
       GtkStyleContext *context;
 
       context = gtk_widget_get_style_context (GTK_WIDGET (self));
-
       gtk_style_context_add_class (context, "timed");
 
-      /* XXX: hardcoding these values isn't really great... */
       if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
-        gtk_widget_set_margin_start (self->main_grid, 8);
+        {
+          gtk_widget_set_margin_start (self->stack, 0);
+          gtk_widget_set_margin_end (self->stack, 2);
+        }
+      else
+        {
+          gtk_widget_set_visible (self->color_box, FALSE);
+        }
     }
 }
 
@@ -322,8 +352,8 @@ gcal_event_widget_set_event_tooltip (GcalEventWidget *self,
             {
               if (is_ltr)
                 {
-                  start = g_date_time_format (tooltip_start, "%x I:%M %P");
-                  end = g_date_time_format (tooltip_end, "%x I:%M %P");
+                  start = g_date_time_format (tooltip_start, "%x %I:%M %P");
+                  end = g_date_time_format (tooltip_end, "%x %I:%M %P");
                 }
               else
                 {
@@ -351,8 +381,8 @@ gcal_event_widget_set_event_tooltip (GcalEventWidget *self,
             {
               if (is_ltr)
                 {
-                  start = g_date_time_format (tooltip_start, "%x, I:%M %P");
-                  end = g_date_time_format (tooltip_end, "I:%M %P");
+                  start = g_date_time_format (tooltip_start, "%x, %I:%M %P");
+                  end = g_date_time_format (tooltip_end, "%I:%M %P");
                 }
               else
                 {
@@ -546,9 +576,12 @@ gcal_event_widget_size_allocate (GtkWidget     *widget,
                                  GtkAllocation *allocation)
 {
   GcalEventWidget *self;
+  GtkAllocation old_allocation;
 
   self = GCAL_EVENT_WIDGET (widget);
 
+  gtk_widget_get_allocation (widget, &old_allocation);
+
   if (gtk_widget_get_realized (widget))
     {
       gdk_window_move_resize (self->event_window,
@@ -558,31 +591,20 @@ gcal_event_widget_size_allocate (GtkWidget     *widget,
                               allocation->height);
     }
 
-  /* Try to put the summary label below if the orientation is vertical */
-  if (self->orientation == GTK_ORIENTATION_VERTICAL && !gcal_event_get_all_day (self->event))
-    {
-      gint minimum_grid_height;
-      gint current_top;
-
-      gtk_widget_get_preferred_height (self->ghost_grid, &minimum_grid_height, NULL);
-
-      gtk_container_child_get (GTK_CONTAINER (self->main_grid),
-                               self->summary_label,
-                               "top-attach", &current_top,
-                               NULL);
-
-      /*
-       * Since we can't change the position of the events inside size-allocate,
-       * queue it for future iterations of the mainloop.
-       */
-      if ((allocation->height < minimum_grid_height && current_top == 1) ||
-          (allocation->height >= minimum_grid_height && current_top == 0))
-        {
-          queue_toggle_description_position (self);
-        }
-    }
-
   GTK_WIDGET_CLASS (gcal_event_widget_parent_class)->size_allocate (widget, allocation);
+
+  /*
+   * Only after the child widgets (main grid, labels, etc) are allocated with the parent
+   * class' allocation function, we can check if the current height is enough to hold the
+   * vertical labels.
+   */
+  if (old_allocation.width != allocation->width || old_allocation.height != allocation->height)
+    {
+      if (self->orientation == GTK_ORIENTATION_HORIZONTAL || gcal_event_get_all_day (self->event))
+        return;
+
+      queue_set_vertical_labels (self, can_hold_vertical_labels_with_height (self, allocation->height));
+    }
 }
 
 static cairo_surface_t*
@@ -608,8 +630,16 @@ static void
 gcal_event_widget_drag_begin (GtkWidget      *widget,
                               GdkDragContext *context)
 {
-  GcalEventWidget *self = GCAL_EVENT_WIDGET (widget);
+  GcalEventWidget *self;
   cairo_surface_t *surface;
+  GdkWindow *window;
+  GdkDevice *device;
+  gint x, y;
+  GtkAllocation allocation;
+
+  self = GCAL_EVENT_WIDGET (widget);
+  window = gtk_widget_get_window (widget);
+  device = gdk_drag_context_get_device (context);
 
   if (self->read_only)
     {
@@ -622,6 +652,11 @@ gcal_event_widget_drag_begin (GtkWidget      *widget,
 
   gtk_drag_set_icon_surface (context, surface);
 
+  /* reposition drag surface to the point the GCalEvent was */
+  gtk_widget_get_allocation (widget, &allocation);
+  gdk_window_get_device_position (window, device, &x, &y, NULL);
+  gdk_drag_context_set_hotspot (context, x - allocation.x, y - allocation.y);
+
   g_clear_pointer (&surface, cairo_surface_destroy);
 }
 
@@ -748,8 +783,8 @@ gcal_event_widget_set_property (GObject      *object,
 
     case PROP_ORIENTATION:
       self->orientation = g_value_get_enum (value);
+      gtk_widget_set_visible (self->vertical_grid, self->orientation == GTK_ORIENTATION_VERTICAL);
       gcal_event_widget_update_style (self);
-      gtk_widget_queue_allocate (GTK_WIDGET (object));
       g_object_notify (object, "orientation");
       break;
 
@@ -804,11 +839,11 @@ gcal_event_widget_finalize (GObject *object)
   g_clear_pointer (&self->css_class, g_free);
   g_clear_object (&self->event);
 
-  /* withdraw mainloop sources */
-  if (self->toggle_description_id > 0)
+  /* remove timeouts */
+  if (self->vertical_label_source_id > 0)
     {
-      g_source_remove (self->toggle_description_id);
-      self->toggle_description_id = 0;
+      g_source_remove (self->vertical_label_source_id);
+      self->vertical_label_source_id = 0;
     }
 
   G_OBJECT_CLASS (gcal_event_widget_parent_class)->finalize (object);
@@ -900,10 +935,12 @@ gcal_event_widget_class_init (GcalEventWidgetClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/event-widget.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, ghost_grid);
+  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, color_box);
+  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, horizontal_grid);
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, hour_label);
-  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, main_grid);
+  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, stack);
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, summary_label);
+  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, vertical_grid);
 
   gtk_widget_class_set_css_name (widget_class, "event");
 }
@@ -1131,11 +1168,11 @@ gint
 gcal_event_widget_sort_events (GcalEventWidget *widget1,
                                GcalEventWidget *widget2)
 {
-  g_autoptr (GDateTime) dt_time1, dt_time2;
+  g_autoptr (GDateTime) dt_time1 = NULL;
+  g_autoptr (GDateTime) dt_time2 = NULL;
   icaltimetype *ical_dt;
   gint diff;
 
-  dt_time1 = dt_time2 = NULL;
   diff = gcal_event_is_multiday (widget2->event) - gcal_event_is_multiday (widget1->event);
 
   if (diff != 0)
@@ -1152,10 +1189,12 @@ gcal_event_widget_sort_events (GcalEventWidget *widget1,
     return diff;
 
   e_cal_component_get_last_modified (gcal_event_get_component (widget1->event), &ical_dt);
-  dt_time1 = icaltime_to_datetime (ical_dt);
+  if (ical_dt)
+    dt_time1 = icaltime_to_datetime (ical_dt);
 
   e_cal_component_get_last_modified (gcal_event_get_component (widget2->event), &ical_dt);
-  dt_time2 = icaltime_to_datetime (ical_dt);
+  if (ical_dt)
+    dt_time2 = icaltime_to_datetime (ical_dt);
 
-  return g_date_time_compare (dt_time2, dt_time1);
+  return dt_time1 && dt_time2 ? g_date_time_compare (dt_time2, dt_time1) : 0;
 }
