@@ -49,18 +49,12 @@ struct _GcalWeekGrid
   GtkContainer        parent;
 
   GtkWidget          *hours_sidebar;
-
   GdkWindow          *event_window;
 
-  gint                first_weekday;
-  gboolean            use_24h_format : 1;
-
   icaltimetype       *active_date;
 
   GcalRangeTree      *events;
 
-  gboolean            children_changed;
-
   /*
    * These fields are "cells" rather than minutes. Each cell
    * correspond to 30 minutes.
@@ -618,14 +612,6 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
   guint x, y;
   guint i;
 
-  /* No need to relayout stuff if nothing changed */
-  if (allocation->height == gtk_widget_get_allocated_height (widget) &&
-      allocation->width == gtk_widget_get_allocated_width (widget) &&
-      !self->children_changed)
-    {
-      return;
-    }
-
   /* Allocate the widget */
   gtk_widget_set_allocation (widget, allocation);
 
@@ -731,8 +717,6 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
       g_clear_pointer (&widgets_data, g_ptr_array_unref);
     }
 
-  self->children_changed = FALSE;
-
   g_clear_pointer (&overlaps, gcal_range_tree_unref);
 }
 
@@ -1157,24 +1141,6 @@ gcal_week_grid_set_manager (GcalWeekGrid *self,
                             self);
 }
 
-void
-gcal_week_grid_set_first_weekday (GcalWeekGrid *self,
-                                  gint          nr_day)
-{
-  g_return_if_fail (GCAL_IS_WEEK_GRID (self));
-
-  self->first_weekday = nr_day;
-}
-
-void
-gcal_week_grid_set_use_24h_format (GcalWeekGrid *self,
-                                     gboolean    use_24h_format)
-{
-  g_return_if_fail (GCAL_IS_WEEK_GRID (self));
-
-  self->use_24h_format = use_24h_format;
-}
-
 void
 gcal_week_grid_add_event (GcalWeekGrid *self,
                           GcalEvent    *event)
@@ -1187,13 +1153,13 @@ gcal_week_grid_add_event (GcalWeekGrid *self,
   end = 0;
   start = 0;
 
+  g_object_ref (event);
+
   widget = g_object_new (GCAL_TYPE_EVENT_WIDGET,
                          "event", event,
                          "orientation", GTK_ORIENTATION_VERTICAL,
                          NULL);
 
-  self->children_changed = TRUE;
-
   get_event_range (self, event, &start, &end);
 
   gcal_range_tree_add_range (self->events,
@@ -1231,13 +1197,12 @@ gcal_week_grid_remove_event (GcalWeekGrid *self,
       if (g_strcmp0 (gcal_event_get_uid (event), uid) != 0)
         continue;
 
-      self->children_changed = TRUE;
-
       get_event_range (self, event, &event_start, &event_end);
 
       gcal_range_tree_remove_range (self->events, data->start, data->end, data);
       destroy_event_widget (self, data->widget);
       gtk_widget_queue_allocate (GTK_WIDGET (self));
+      g_object_unref (event);
       g_free (data);
     }
 
