@@ -34,47 +34,69 @@
 
 #define COLUMN_PADDING       6
 
+/* WeatherInfoDay:
+ * @winfo: (nullable): Holds weather information for this week-day. All other fields are only valid if this one is not %NULL.
+ * @icon_buf: (nullable): Buffered weather icon.
+ * @x: X-position of weather indicators.
+ * @y: Y-positon of weather indicators.
+ * @width: Width of weather indicators.
+ * @height: Height of weather indicators.
+ *
+ * Structure represents a weather indicator for a single day.
+ *
+ * Location information are stored for mouse-over events.
+ */
+typedef struct
+{
+  GcalWeatherInfo    *winfo;    /* owned */
+  GdkPixbuf          *icon_buf; /* owned */
+} WeatherInfoDay;
+
+
+
 struct _GcalWeekHeader
 {
-  GtkGrid           parent;
+  GtkGrid             parent;
 
-  GtkWidget        *grid;
-  GtkWidget        *month_label;
-  GtkWidget        *week_label;
-  GtkWidget        *year_label;
-  GtkWidget        *scrolledwindow;
-  GtkWidget        *expand_button;
-  GtkWidget        *expand_button_box;
-  GtkWidget        *expand_button_image;
-  GtkWidget        *header_labels_box;
+  GtkWidget          *grid;
+  GtkWidget          *month_label;
+  GtkWidget          *week_label;
+  GtkWidget          *year_label;
+  GtkWidget          *scrolledwindow;
+  GtkWidget          *expand_button;
+  GtkWidget          *expand_button_box;
+  GtkWidget          *expand_button_image;
+  GtkWidget          *header_labels_box;
 
-  GcalManager      *manager;
+  GcalManager        *manager;
 
   /*
    * Stores the events as they come from the week-view
    * The list will later be iterated after the active date is changed
    * and the events will be placed
    */
-  GList            *events[7];
-  GtkWidget        *overflow_label[7];
+  GList              *events[7];
+  GtkWidget          *overflow_label[7];
 
-  gint              first_weekday;
+  gint                first_weekday;
 
   /*
    * Used for checking if the header is in collapsed state or expand state
    * false is collapse state true is expand state
    */
-  gboolean          expanded;
+  gboolean            expanded;
 
-  gboolean          use_24h_format;
+  icaltimetype       *active_date;
 
-  icaltimetype     *active_date;
+  gint                selection_start;
+  gint                selection_end;
+  gint                dnd_cell;
 
-  gint              selection_start;
-  gint              selection_end;
-  gint              dnd_cell;
+  GcalWeatherService *weather_service; /* unowned, nullable */
+  /* Array of nullable weather infos for each day, starting with Sunday. */
+  WeatherInfoDay      weather_infos[7];
 
-  GtkSizeGroup     *sizegroup;
+  GtkSizeGroup       *sizegroup;
 };
 
 typedef enum
@@ -83,6 +105,13 @@ typedef enum
   DOWN
 } MoveDirection;
 
+enum
+{
+  PROP_0,
+  PROP_WEATHER_SERVICE,
+  NUM_PROPS
+};
+
 enum
 {
   EVENT_ACTIVATED,
@@ -93,6 +122,26 @@ static guint signals[LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GcalWeekHeader, gcal_week_header, GTK_TYPE_GRID);
 
+
+/* WeatherInfoDay methods */
+
+/* wid_clear:
+ *
+ * Drops all internal references and resets
+ * geometric information.
+ */
+static inline void
+wid_clear (WeatherInfoDay *wid)
+{
+   g_return_if_fail (wid != NULL);
+
+  if (wid->winfo != NULL)
+    g_clear_object (&wid->winfo);
+
+  if (wid->icon_buf != NULL)
+    g_clear_object (&wid->icon_buf);
+}
+
 /* Event activation methods */
 static void
 on_event_widget_activated (GcalEventWidget *widget,
@@ -231,6 +280,17 @@ on_button_released (GcalWeekHeader *self,
   return GDK_EVENT_STOP;
 }
 
+static void
+on_weather_update (GcalWeatherService *weather_service,
+                   GcalWeekHeader     *self)
+{
+  g_return_if_fail (GCAL_IS_WEATHER_SERVICE (weather_service));
+  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
+  g_return_if_fail (self->weather_service == weather_service);
+
+  gcal_week_header_update_weather_infos (self);
+}
+
 static GcalEvent*
 get_event_by_uuid (GcalWeekHeader *self,
                    const gchar    *uuid)
@@ -663,6 +723,9 @@ add_event_to_grid (GcalWeekHeader *self,
   gint position;
   gint i;
 
+  /* Take a reference to the event */
+  g_object_ref (event);
+
   /* Add at least at the first weekday */
   position = add_event_to_weekday (self, event, start);
 
@@ -1031,24 +1094,57 @@ gcal_week_header_finalize (GObject *object)
 
   for (i = 0; i < 7; i++)
     g_list_free (self->events[i]);
+
+  if (self->weather_service != NULL)
+    {
+      g_signal_handlers_disconnect_by_func (self->weather_service,
+                                            (GCallback) on_weather_update,
+                                            self);
+      g_clear_object (&self->weather_service);
+    }
+
+  for (i = 0; i < G_N_ELEMENTS (self->weather_infos); i++)
+    wid_clear (&self->weather_infos[i]);
 }
 
 static void
 gcal_week_header_get_property (GObject    *object,
-                               guint       prop_id,
+                               guint       property_id,
                                GValue     *value,
                                GParamSpec *pspec)
 {
-  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  GcalWeekHeader *self = GCAL_WEEK_HEADER (object);
+
+  switch (property_id)
+    {
+    case PROP_WEATHER_SERVICE:
+      g_value_set_boxed (value, self->weather_service);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+    }
 }
 
 static void
 gcal_week_header_set_property (GObject      *object,
-                               guint         prop_id,
+                               guint         property_id,
                                const GValue *value,
                                GParamSpec   *pspec)
 {
-  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  GcalWeekHeader *self = GCAL_WEEK_HEADER (object);
+
+  switch (property_id)
+    {
+    case PROP_WEATHER_SERVICE:
+        gcal_week_header_set_weather_service (self, GCAL_WEATHER_SERVICE (g_value_get_object (value)));
+        break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+    }
 }
 
 static void
@@ -1082,7 +1178,7 @@ gcal_week_header_draw (GtkWidget      *widget,
   PangoFontDescription *bold_font;
 
   gdouble cell_width;
-  gint i, font_height, current_cell, today_column;
+  gint i, day_abv_font_height, current_cell, today_column;
   gint start_x, start_y;
 
   gboolean ltr;
@@ -1174,12 +1270,14 @@ gcal_week_header_draw (GtkWidget      *widget,
       gtk_style_context_restore (context);
     }
 
-  pango_layout_get_pixel_size (layout, NULL, &font_height);
+  pango_layout_get_pixel_size (layout, NULL, &day_abv_font_height);
 
   for (i = 0; i < 7; i++)
     {
+      WeatherInfoDay *wdinfo; /* unowned */
       gchar *weekday_date, *weekday_abv, *weekday;
       gdouble x;
+      gint day_num_font_height, day_num_font_baseline;
       gint font_width;
       gint n_day;
 
@@ -1201,7 +1299,8 @@ gcal_week_header_draw (GtkWidget      *widget,
       pango_layout_set_font_description (layout, bold_font);
       pango_layout_set_text (layout, weekday_date, -1);
 
-      pango_layout_get_pixel_size (layout, &font_width, NULL);
+      pango_layout_get_pixel_size (layout, &font_width, &day_num_font_height);
+      day_num_font_baseline = pango_layout_get_baseline (layout) / PANGO_SCALE;
 
       if (ltr)
         x = padding.left + cell_width * i + COLUMN_PADDING + start_x;
@@ -1211,14 +1310,14 @@ gcal_week_header_draw (GtkWidget      *widget,
       gtk_render_layout (context,
                          cr,
                          x,
-                         font_height + padding.bottom + start_y,
+                         day_abv_font_height + padding.bottom + start_y,
                          layout);
 
       gtk_style_context_restore (context);
 
       /* Draws the days name */
       weekday = g_utf8_strup (gcal_get_weekday ((i + self->first_weekday) % 7), -1);
-      weekday_abv = g_strdup_printf ("%s", weekday);
+      weekday_abv = g_strdup (weekday);
       g_free (weekday);
 
       gtk_style_context_save (context);
@@ -1246,6 +1345,84 @@ gcal_week_header_draw (GtkWidget      *widget,
 
       gtk_style_context_restore (context);
 
+      /* Draws weather icon if given */
+      wdinfo = &self->weather_infos[ltr? i : 6 - i];
+      if (wdinfo->winfo != NULL)
+        {
+          const gchar *weather_icon_name = gcal_weather_info_get_icon_name (wdinfo->winfo);
+          const gchar *weather_temp = gcal_weather_info_get_temperature (wdinfo->winfo);
+
+          /* Imagine a box around weather indicators with length MAX(imgW,tempW)
+           * We compute its width and position in this section.
+           * Its height is derived from day name and numbers. The icon sticks on
+           * the very top and the temperature on its base.
+           */
+          gint icon_size = day_num_font_height - 3; /* available space - space between temperature and icon */
+          gint temp_width = 0;
+          gint temp_height = 0;
+          gint temp_baseline = 0;
+
+          gdouble wibox_width;
+          gdouble wibox_x;
+
+          if (weather_icon_name != NULL && wdinfo->icon_buf == NULL)
+            {
+              GtkIconTheme *theme; /* unowned */
+              g_autoptr (GError) err = NULL;
+              gint icon_flags;
+
+              theme = gtk_icon_theme_get_default ();
+              /* TODO: catch icon theme changes */
+
+              icon_flags = ltr? GTK_ICON_LOOKUP_FORCE_SIZE | GTK_ICON_LOOKUP_DIR_LTR
+                              : GTK_ICON_LOOKUP_FORCE_SIZE | GTK_ICON_LOOKUP_DIR_RTL;
+              wdinfo->icon_buf = gtk_icon_theme_load_icon (theme, weather_icon_name, icon_size, icon_flags, &err);
+              if (err != NULL)
+                {
+                  g_assert (wdinfo->icon_buf == NULL);
+                  g_warning ("Could not load icon %s: %s", weather_icon_name, err->message);
+                }
+            }
+
+          if (G_LIKELY (weather_temp != NULL))
+            {
+              gtk_style_context_save (context);
+              gtk_style_context_add_class (context, "week-temperature");
+              gtk_style_context_get (context, state, "font", &bold_font, NULL);
+
+              pango_layout_set_font_description (layout, bold_font);
+              pango_layout_set_text (layout, weather_temp, -1);
+
+              pango_layout_get_pixel_size (layout, &temp_width, &temp_height);
+              temp_baseline = pango_layout_get_baseline (layout) / PANGO_SCALE;
+            }
+
+          wibox_width = MAX(icon_size, temp_width);
+          wibox_x = ltr ? alloc.width - (cell_width * (6 - i) + wibox_width + COLUMN_PADDING)
+                        : padding.left + cell_width * i + COLUMN_PADDING + start_x;
+
+          /* Actually draw weather indicator: */
+          if (G_LIKELY (weather_temp != NULL))
+            {
+              gtk_render_layout (context,
+                                 cr,
+                                 wibox_x + (wibox_width - temp_width) / 2,
+                                 day_abv_font_height + padding.bottom + start_y + day_num_font_baseline - temp_baseline,
+                                 layout);
+              gtk_style_context_restore (context);
+            }
+
+          if (G_LIKELY (wdinfo->icon_buf != NULL))
+            {
+              gdk_cairo_set_source_pixbuf (cr,
+                                           wdinfo->icon_buf,
+                                           wibox_x + (wibox_width - icon_size) / 2,
+                                           start_y);
+              cairo_paint (cr);
+            }
+        }
+
+
       /* Draws the lines after each day of the week */
       gtk_style_context_save (context);
       gtk_style_context_add_class (context, "lines");
@@ -1257,11 +1434,11 @@ gcal_week_header_draw (GtkWidget      *widget,
 
       cairo_move_to (cr,
                      ALIGNED (ltr ? (cell_width * i + start_x) : (alloc.width - (cell_width * i + start_x))),
-                     font_height + padding.bottom + start_y);
+                     day_abv_font_height + padding.bottom + start_y);
 
       cairo_rel_line_to (cr,
                          0.0,
-                         gtk_widget_get_allocated_height (widget) - font_height - start_y + padding.bottom);
+                         gtk_widget_get_allocated_height (widget) - day_abv_font_height - start_y + padding.bottom);
       cairo_stroke (cr);
 
       gtk_style_context_restore (context);
@@ -1407,7 +1584,7 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
     gcal_event_set_all_day (event, TRUE);
 
   /* Commit the changes */
-  gcal_manager_update_event (self->manager, event, E_CAL_OBJ_MOD_THIS);
+  gcal_manager_update_event (self->manager, event, GCAL_RECURRENCE_MOD_THIS_ONLY);
 
   /* Cancel the DnD */
   self->dnd_cell = -1;
@@ -1448,6 +1625,19 @@ gcal_week_header_class_init (GcalWeekHeaderClass *kclass)
   widget_class->drag_leave = gcal_week_header_drag_leave;
   widget_class->drag_drop = gcal_week_header_drag_drop;
 
+  /**
+   * GcalWeekHeader::weather-service:
+   *
+   * The #GcalWeatherService of the view.
+   */
+  g_object_class_install_property (G_OBJECT_CLASS (kclass),
+                                   PROP_WEATHER_SERVICE,
+                                   g_param_spec_object ("weather-service",
+                                                        "The weather service",
+                                                        "The weather service of the view",
+                                                        GCAL_TYPE_WEATHER_SERVICE,
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   signals[EVENT_ACTIVATED] = g_signal_new ("event-activated",
                                            GCAL_TYPE_WEEK_HEADER,
                                            G_SIGNAL_RUN_FIRST,
@@ -1484,6 +1674,7 @@ gcal_week_header_init (GcalWeekHeader *self)
   self->selection_start = -1;
   self->selection_end = -1;
   self->dnd_cell = -1;
+  self->first_weekday = get_first_weekday ();
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
@@ -1500,7 +1691,127 @@ G_GNUC_END_IGNORE_DEPRECATIONS
                      GDK_ACTION_MOVE);
 }
 
+/* Private API */
+
+/* gcal_week_header_add_weather_infos:
+ * @self:  (not nullable): The #GcalWeekHeader instance.
+ * @winfo: List of #GcalWeatherInfos to add to @self.
+ *
+ * Adds weather information to this header. @self
+ * only adds weather information that can be
+ * displayed with current settings.
+ *
+ * Return: Number of consumed weather information
+ *         objects.
+ */
+static gint
+gcal_week_header_add_weather_infos (GcalWeekHeader *self,
+                                    GPtrArray      *weather_infos)
+{
+  g_autoptr (GDateTime) week_start_dt = NULL;
+  GDate week_start;
+  gint consumed = 0;
+  guint i;
+
+  if (!self->active_date)
+    return 0;
+
+  week_start_dt = get_start_of_week (self->active_date);
+  g_date_set_dmy (&week_start,
+                  g_date_time_get_day_of_month (week_start_dt),
+                  g_date_time_get_month (week_start_dt),
+                  g_date_time_get_year (week_start_dt));
+
+  for (i = 0; weather_infos && i < weather_infos->len; i++)
+    {
+      GcalWeatherInfo *gwi; /* unowned */
+      GDate gwi_date;
+      gint day_diff;
+
+      gwi = g_ptr_array_index (weather_infos, i);
+
+      gcal_weather_info_get_date (gwi, &gwi_date);
+
+      day_diff = g_date_days_between (&week_start, &gwi_date);
+      if (day_diff >= 0 && day_diff < G_N_ELEMENTS (self->weather_infos))
+        {
+          wid_clear (&self->weather_infos[day_diff]);
+          self->weather_infos[day_diff].winfo = g_object_ref (gwi);
+          consumed++;
+        }
+    }
+
+  if (consumed > 0)
+    gtk_widget_queue_draw (GTK_WIDGET (self));
+
+  return consumed;
+}
+
+/* gcal_week_header_clear_weather_infos:
+ * @self: The #GcalWeekHeader instance.
+ *
+ * Removes all registered weather information objects from
+ * this widget.
+ */
+static void
+gcal_week_header_clear_weather_infos (GcalWeekHeader *self)
+{
+  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
+
+  for (gint i = 0; i < G_N_ELEMENTS (self->weather_infos); i++)
+    wid_clear (&self->weather_infos[i]);
+
+  gtk_widget_queue_draw (GTK_WIDGET (self));
+}
+
+
 /* Public API */
+
+/**
+ * gcal_week_header_set_weather_service:
+ * @self:    The #GcalWeekHeader instance.
+ * @service: (nullable): The weather service to query.
+ *
+ * Note that #GcalWeekHeader does not hold a strong reference
+ * to its weather service.
+ */
+void
+gcal_week_header_set_weather_service (GcalWeekHeader     *self,
+                                      GcalWeatherService *service)
+{
+  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
+  g_return_if_fail (service == NULL || GCAL_IS_WEATHER_SERVICE (service));
+
+  gcal_view_set_weather_service_impl_helper (&self->weather_service,
+                                             service,
+                                             (GcalWeatherUpdateFunc) gcal_week_header_update_weather_infos,
+                                             (GCallback) on_weather_update,
+                                             GTK_WIDGET (self));
+}
+
+/**
+ * gcal_week_header_update_weather_infos:
+ * @self: The #GcalWeekHeader instance.
+ *
+ * Retrieves latest weather information from registered
+ * weather service and displays it.
+ */
+void
+gcal_week_header_update_weather_infos (GcalWeekHeader *self)
+{
+  GPtrArray* weather_infos;
+
+  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
+
+  gcal_week_header_clear_weather_infos (self);
+
+  if (!self->weather_service)
+    return;
+
+  weather_infos = gcal_weather_service_get_weather_infos (self->weather_service);
+  gcal_week_header_add_weather_infos (self, weather_infos);
+}
+
 void
 gcal_week_header_set_manager (GcalWeekHeader *self,
                               GcalManager    *manager)
@@ -1515,24 +1826,6 @@ gcal_week_header_set_manager (GcalWeekHeader *self,
                             self);
 }
 
-void
-gcal_week_header_set_first_weekday (GcalWeekHeader *self,
-                                    gint            nr_day)
-{
-  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
-
-  self->first_weekday = nr_day;
-}
-
-void
-gcal_week_header_set_use_24h_format (GcalWeekHeader *self,
-                                     gboolean        use_24h_format)
-{
-  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
-
-  self->use_24h_format = use_24h_format;
-}
-
 void
 gcal_week_header_add_event (GcalWeekHeader *self,
                             GcalEvent      *event)
@@ -1619,7 +1912,7 @@ void
 gcal_week_header_remove_event (GcalWeekHeader *self,
                                const gchar    *uuid)
 {
-  GcalEvent *removed_event;
+  g_autoptr (GcalEvent) removed_event = NULL;
   GList *children, *l;
   gint weekday;
 
@@ -1677,32 +1970,20 @@ gcal_week_header_remove_event (GcalWeekHeader *self,
 }
 
 GList*
-gcal_week_header_get_children_by_uuid (GcalWeekHeader *self,
-                                       const gchar    *uuid)
+gcal_week_header_get_children_by_uuid (GcalWeekHeader        *self,
+                                       GcalRecurrenceModType  mod,
+                                       const gchar           *uuid)
 {
-  GList *children, *l, *result;
+  GList *children, *result;
+
+  GCAL_ENTRY;
 
-  result = NULL;
   children = gtk_container_get_children (GTK_CONTAINER (self->grid));
-
-  for (l = children; l != NULL; l = l->next)
-    {
-      GcalEventWidget *child_widget;
-      GcalEvent *event;
-
-      if (!GCAL_IS_EVENT_WIDGET (l->data))
-        continue;
-
-      child_widget = GCAL_EVENT_WIDGET (l->data);
-      event = gcal_event_widget_get_event (child_widget);
-
-      if (g_strcmp0 (uuid, gcal_event_get_uid (event)) == 0)
-        result = g_list_prepend (result, l->data);
-    }
+  result = filter_event_list_by_uid_and_modtype (children, mod, uuid);
 
   g_list_free (children);
 
-  return result;
+  GCAL_RETURN (result);
 }
 
 GtkSizeGroup*
@@ -1753,5 +2034,32 @@ gcal_week_header_set_date (GcalWeekHeader *self,
   if (old_date)
     update_unchanged_events (self, self->active_date);
 
+  gcal_week_header_update_weather_infos (self);
+
   g_clear_pointer (&old_date, g_free);
 }
+
+/**
+ * gcal_week_header_get_weather_infos:
+ * @self: The #GcalWeekHeader instance.
+ *
+ * Returns a list of shown weather informations.
+ *
+ * @Returns: (transfer container):
+ *           A GSList. The callee is responsible for freeing it.
+ *           Elements are owned by the widget. Do not modify.
+ */
+GSList*
+gcal_week_header_get_shown_weather_infos (GcalWeekHeader *self)
+{
+  g_return_val_if_fail (GCAL_IS_WEEK_HEADER (self), NULL);
+  GSList* lst = NULL; /* owned[unowned] */
+
+  for (int i = 0; i < G_N_ELEMENTS (self->weather_infos); i++)
+    {
+      if (self->weather_infos[i].winfo != NULL)
+        lst = g_slist_prepend (lst, self->weather_infos[i].winfo);
+    }
+
+   return lst;
+}
