@@ -1,4 +1,3 @@
-/* -*- mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* gcal-year-view.c
  *
  * Copyright (C) 2015 Erick Pérez Castellanos <erick.red@gmail.com>
@@ -35,106 +34,100 @@
 typedef struct
 {
   /* month span from 0 to 11 */
-  gint start_day, start_month;
-  gint end_day, end_month;
-  gint hovered_day, hovered_month;
-  gint dnd_day, dnd_month;
+  gint                start_day, start_month;
+  gint                end_day, end_month;
+  gint                hovered_day, hovered_month;
+  gint                dnd_day, dnd_month;
 } ButtonData;
 
 typedef struct
 {
-  gdouble  box_side;
-  GdkPoint coordinates [12];
+  gdouble             box_side;
+  GdkPoint            coordinates [12];
 } GridData;
 
 struct _GcalYearView
 {
-  GtkBox        parent;
+  GtkBox              parent;
 
   /* composite, GtkBuilder's widgets */
-  GtkWidget    *navigator;
-  GtkWidget    *sidebar;
-  GtkWidget    *events_sidebar;
-  GtkWidget    *navigator_stack;
-  GtkWidget    *no_events_title;
-  GtkWidget    *navigator_sidebar;
-  GtkWidget    *scrolled_window;
+  GtkWidget          *navigator;
+  GtkWidget          *sidebar;
+  GtkWidget          *events_sidebar;
+  GtkWidget          *navigator_stack;
+  GtkWidget          *no_events_title;
+  GtkWidget          *navigator_sidebar;
+  GtkWidget          *scrolled_window;
+  GtkLabel           *temp_label;   /* unowned */
+  GtkImage           *weather_icon; /* unowned */
 
-  GtkWidget    *popover; /* Popover for popover_mode */
+  GtkWidget          *popover; /* Popover for popover_mode */
 
   /* manager singleton */
-  GcalManager  *manager;
+  GcalManager        *manager;
+  GcalWeatherService *weather_service; /* owned, nullable */
 
   /* range shown on the sidebar */
-  icaltimetype *start_selected_date;
-  icaltimetype *end_selected_date;
+  icaltimetype       *start_selected_date;
+  icaltimetype       *end_selected_date;
 
   /* geometry info */
-  GridData     *navigator_grid;
-  guint         number_of_columns;
-  guint         column_width;
-  guint         row_height;
-  guint         header_height;
-  guint         sidebar_width;
+  GridData           *navigator_grid;
+  guint               number_of_columns;
+  guint               column_width;
+  guint               row_height;
+  guint               header_height;
+  guint               sidebar_width;
 
   /* state flags */
-  gboolean      popover_mode;
-  gboolean      button_pressed;
-  ButtonData   *selected_data;
+  gboolean            popover_mode;
+  gboolean            button_pressed;
+  ButtonData         *selected_data;
 
   /**
    * first day of the week according to user locale, being
-   * 0 for Sunday, 1 for Monday and so on */
-  gint          first_weekday;
+   * 0 for Sunday, 1 for Monday and so on
+   */
+  gint                first_weekday;
 
   /* first and last weeks of the year */
-  guint         first_week_of_year;
-  guint         last_week_of_year;
+  guint               first_week_of_year;
+  guint               last_week_of_year;
 
   /* Storage for the accumulated scrolling */
-  gdouble         scroll_value;
-
-    /**
-   * clock format from GNOME desktop settings
-   */
-  gboolean      use_24h_format;
+  gdouble               scroll_value;
 
   /* show week numbers from GNOME Shell settings */
-  GSettings    *calendar_settings;
-  gboolean      show_week_numbers;
+  GSettings          *calendar_settings;
+  gboolean            show_week_numbers;
 
   /* text direction factors */
-  gint          k;
+  gint                k;
 
   /* date property */
-  icaltimetype *date;
+  icaltimetype       *date;
 
   /*
    * Array with the events at every month. Events
    * that span multiple months are added multiple
    * times to the array.
    */
-  GPtrArray    *events[12];
+  GPtrArray          *events[12];
 };
 
 enum {
   PROP_0,
   PROP_DATE,
   PROP_MANAGER,
+  PROP_WEATHER_SERVICE,
   PROP_SHOW_WEEK_NUMBERS,
   LAST_PROP
 };
 
-enum
-{
-  EVENT_ACTIVATED,
-  NUM_SIGNALS
-};
+static void          gcal_view_interface_init                    (GcalViewInterface  *iface);
+static void          gcal_data_model_subscriber_interface_init   (ECalDataModelSubscriberInterface *iface);
+static void          update_weather                              (GcalYearView       *self);
 
-static guint signals[NUM_SIGNALS] = { 0, };
-
-static void   gcal_view_interface_init (GcalViewInterface *iface);
-static void   gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GcalYearView, gcal_year_view, GTK_TYPE_BOX,
                          G_IMPLEMENT_INTERFACE (GCAL_TYPE_VIEW, gcal_view_interface_init)
@@ -176,7 +169,17 @@ event_activated (GcalEventWidget *widget,
 
   if (view->popover_mode)
     gtk_widget_hide (view->popover);
-  g_signal_emit (GCAL_YEAR_VIEW (user_data), signals[EVENT_ACTIVATED], 0, widget);
+  g_signal_emit_by_name (view, "event-activated", widget);
+}
+
+static void
+weather_changed (GcalWeatherService *weather_service,
+                 GcalYearView       *self)
+{
+  g_return_if_fail (GCAL_IS_WEATHER_SERVICE (weather_service));
+  g_return_if_fail (GCAL_IS_YEAR_VIEW (self));
+
+  update_weather (self);
 }
 
 static void
@@ -293,6 +296,7 @@ update_no_events_page (GcalYearView *year_view)
     }
 
   gtk_label_set_text (GTK_LABEL (year_view->no_events_title), title);
+  update_weather (year_view);
   g_free (title);
 }
 
@@ -1603,6 +1607,7 @@ gcal_year_view_finalize (GObject *object)
   g_clear_pointer (&year_view->date, g_free);
 
   g_clear_object (&year_view->calendar_settings);
+  g_clear_object (&year_view->weather_service);
 
   for (i = 0; i < 12; i++)
     g_clear_pointer (&year_view->events[i], g_ptr_array_unref);
@@ -1628,6 +1633,10 @@ gcal_year_view_get_property (GObject    *object,
       g_value_set_object (value, self->manager);
       break;
 
+    case PROP_WEATHER_SERVICE:
+      g_value_set_object (value, self->weather_service);
+      break;
+
     case PROP_SHOW_WEEK_NUMBERS:
       g_value_set_boolean (value, self->show_week_numbers);
       break;
@@ -1662,6 +1671,14 @@ gcal_year_view_set_property (GObject      *object,
       g_object_notify (object, "manager");
       break;
 
+    case PROP_WEATHER_SERVICE:
+      gcal_view_set_weather_service_impl_helper (&self->weather_service,
+                                                 g_value_get_object (value),
+                                                 (GcalWeatherUpdateFunc) update_weather,
+                                                 (GCallback) weather_changed,
+                                                 GTK_WIDGET (self));
+      break;
+
     case PROP_SHOW_WEEK_NUMBERS:
       if (self->show_week_numbers != g_value_get_boolean (value))
         {
@@ -1783,22 +1800,27 @@ gcal_year_view_component_added (ECalDataModelSubscriber *subscriber,
                                 ECalClient              *client,
                                 ECalComponent           *comp)
 {
-  GcalYearView *self = GCAL_YEAR_VIEW (subscriber);
+  g_autoptr (GcalEvent) event = NULL;
+  GcalYearView *self;
   GDateTime *event_start, *event_end;
-  GcalEvent *event;
   GError *error;
   guint i, start_month, end_month;
 
+  GCAL_ENTRY;
+
   error = NULL;
+  self = GCAL_YEAR_VIEW (subscriber);
   event = gcal_event_new (e_client_get_source (E_CLIENT (client)), comp, &error);
 
   if (error)
     {
       g_warning ("Error creating event: %s", error->message);
       g_clear_error (&error);
-      return;
+      GCAL_RETURN ();
     }
 
+  g_debug ("Caching event '%s' in Year view", gcal_event_get_uid (event));
+
   event_start = gcal_event_get_date_start (event);
   event_end = gcal_event_get_date_end (event);
 
@@ -1819,6 +1841,8 @@ gcal_year_view_component_added (ECalDataModelSubscriber *subscriber,
   update_sidebar (self);
 
   gtk_widget_queue_draw (GTK_WIDGET (self->navigator));
+
+  GCAL_EXIT;
 }
 
 static void
@@ -1827,13 +1851,16 @@ gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
                                   const gchar             *uid,
                                   const gchar             *rid)
 {
-  GcalYearView *year_view = GCAL_YEAR_VIEW (subscriber);
+  GcalYearView *year_view;
   GList *children, *l;
   ESource *source;
-  gchar *uuid;
+  g_autofree gchar *uuid = NULL;
   guint i;
   gint number_of_children;
 
+  GCAL_ENTRY;
+
+  year_view = GCAL_YEAR_VIEW (subscriber);
   source = e_client_get_source (E_CLIENT (client));
   if (rid != NULL)
     uuid = g_strdup_printf ("%s:%s:%s", e_source_get_uid (source), uid, rid);
@@ -1882,9 +1909,10 @@ gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
 
           event = g_ptr_array_index (events, j);
 
-          if (g_strcmp0 (gcal_event_get_uid (event), uuid) != 0)
+          if (!g_str_equal (gcal_event_get_uid (event), uuid))
             continue;
 
+          g_debug ("Removing event '%s' from Year view's cache", uuid);
           g_ptr_array_remove (events, event);
         }
     }
@@ -1892,7 +1920,8 @@ gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
   gtk_widget_queue_draw (GTK_WIDGET (year_view->navigator));
 
   g_list_free (children);
-  g_free (uuid);
+
+  GCAL_EXIT;
 }
 
 static void
@@ -1902,12 +1931,16 @@ gcal_year_view_component_changed (ECalDataModelSubscriber *subscriber,
 {
   ECalComponentId *id;
 
+  GCAL_ENTRY;
+
   id = e_cal_component_get_id (comp);
 
   gcal_year_view_component_removed (subscriber, client, id->uid, id->rid);
   gcal_year_view_component_added (subscriber, client, comp);
 
   g_clear_pointer (&id, e_cal_component_free_id);
+
+  GCAL_EXIT;
 }
 
 static void
@@ -1937,6 +1970,7 @@ gcal_year_view_class_init (GcalYearViewClass *klass)
 
   g_object_class_override_property (object_class, PROP_DATE, "active-date");
   g_object_class_override_property (object_class, PROP_MANAGER, "manager");
+  g_object_class_override_property (object_class, PROP_WEATHER_SERVICE, "weather-service");
 
   g_object_class_install_property (object_class,
                                    PROP_SHOW_WEEK_NUMBERS,
@@ -1946,15 +1980,6 @@ gcal_year_view_class_init (GcalYearViewClass *klass)
                                                          FALSE,
                                                          G_PARAM_READWRITE));
 
-  /* FIXME: it will problably go back to GcalView */
-  signals[EVENT_ACTIVATED] = g_signal_new ("event-activated",
-                                           GCAL_TYPE_YEAR_VIEW,
-                                           G_SIGNAL_RUN_LAST,
-                                           0, NULL, NULL, NULL,
-                                           G_TYPE_NONE,
-                                           1,
-                                           GCAL_TYPE_EVENT_WIDGET);
-
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/year-view.ui");
 
   gtk_widget_class_bind_template_child (widget_class, GcalYearView, navigator);
@@ -1965,6 +1990,8 @@ gcal_year_view_class_init (GcalYearViewClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalYearView, no_events_title);
   gtk_widget_class_bind_template_child (widget_class, GcalYearView, popover);
   gtk_widget_class_bind_template_child (widget_class, GcalYearView, scrolled_window);
+  gtk_widget_class_bind_template_child (widget_class, GcalYearView, temp_label);
+  gtk_widget_class_bind_template_child (widget_class, GcalYearView, weather_icon);
 
   gtk_widget_class_bind_template_callback (widget_class, draw_navigator);
   gtk_widget_class_bind_template_callback (widget_class, navigator_button_press_cb);
@@ -1986,6 +2013,8 @@ gcal_year_view_init (GcalYearView *self)
 {
   guint i;
 
+  self->weather_service = NULL;
+
   for (i = 0; i < 12; i++)
     self->events[i] = g_ptr_array_new_with_free_func (g_object_unref);
 
@@ -1996,6 +2025,8 @@ gcal_year_view_init (GcalYearView *self)
   else if (gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL)
     self->k = 1;
 
+  self->first_weekday = get_first_weekday ();
+
   self->navigator_grid = g_new0 (GridData, 1);
   self->selected_data = g_new0 (ButtonData, 1);
 
@@ -2034,17 +2065,52 @@ gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *ifa
   iface->thaw = gcal_year_view_thaw;
 }
 
-/* Public API */
-void
-gcal_year_view_set_first_weekday (GcalYearView *year_view,
-                                  gint          nr_day)
+static void
+update_weather (GcalYearView *self)
 {
-  year_view->first_weekday = nr_day;
+  gboolean updated = FALSE;
+
+  g_return_if_fail (GCAL_IS_YEAR_VIEW (self));
+
+  if (self->weather_service && self->date)
+    {
+      GPtrArray *weather_infos;
+      GDate date;
+      guint i;
+
+      g_date_set_dmy (&date, self->date->day, self->date->month, self->date->year);
+
+      weather_infos = gcal_weather_service_get_weather_infos (self->weather_service);
+
+      for (i = 0; weather_infos && i < weather_infos->len; i++)
+        {
+          GcalWeatherInfo *info;
+          GDate wdate;
+
+          info = g_ptr_array_index (weather_infos, i);
+
+          gcal_weather_info_get_date (info, &wdate);
+
+          if (g_date_compare (&date, &wdate) == 0)
+            {
+              const gchar *temp_str; /* unowned */
+              const gchar *icon_nm;  /* unowned */
+
+              temp_str = gcal_weather_info_get_temperature (info);
+              icon_nm = gcal_weather_info_get_icon_name (info);
+
+              gtk_label_set_text (self->temp_label, temp_str);
+              gtk_image_set_from_icon_name (self->weather_icon, icon_nm, GTK_ICON_SIZE_SMALL_TOOLBAR);
+              updated = TRUE;
+              break;
+            }
+        }
+    }
+
+  if (!updated)
+    {
+      gtk_label_set_text (self->temp_label, "");
+      gtk_image_clear (self->weather_icon);
+    }
 }
 
-void
-gcal_year_view_set_use_24h_format (GcalYearView *year_view,
-                                   gboolean      use_24h_format)
-{
-  year_view->use_24h_format = use_24h_format;
-}
